# 实验8 一级 Misc —— 传统图片隐写方式简介

## 必做题

### 白与夜

鼠标右键在新标签页打开图像，即可看到 flag

### LSB隐写

运行助教给的 jar 包：

```bash
java -jar Stegsolve.jar
```

打开图片后点击下方的右方向按钮多次即可看到 flag

### 尾巴的秘密

在 linux 中直接 cat 出现乱码导致终端模拟器混乱。首先我猜 cat 应该有很多选项：

```bash
$ tldr cat
（此处省略若干行）               
 - 显示不可打印和空白的字符（使用 M- 前缀标记非 ASCII 字符）：
   cat -v -t -e file
```

执行：

```bash
cat -v -t -e  ~/download/tail_challenge.jpg | tail -n 1 
```

可以获得结尾的内容。

### 盲水印

参见助教给的讲义，打开对应的轮子链接，参考对应内容可以写出对应代码：

```python
bwm1 = WaterMark(password_wm=1, password_img=1)
bwm1.extract(filename='input.png', wm_shape=(26, 340), out_wm_name='output.png', )
```

可以获得该处水印。

### 对上述盲水印实现的鲁棒性做探究

作为举例，我们[在这里](https://suzhou.ustc.edu.cn/2021/0603/c28357a511553/page.htm)下载中科大的校徽：

![1](img/81.jpg)

还可以下载[国科大](https://www.cas.cn/zj/yk/201410/t20141017_4225629.shtml)的校徽：

![2](img/82.jpg)

直接嵌入发现国科大图标太大了，使用 ffmpeg 缩小：

```bash
ffmpeg -i  img/82.png -vf scale=64:-1 img/820.png
```

再次运行，生成 83.png

![3](img/83.png)

```bash
$ file 83.png 
83.png: PNG image data, 638 x 656, 8-bit/color RGB, non-interlaced
```

先看直接提取，作为对照组：

![830](img/830.png)

现在采取攻击：

#### 1. 裁剪

```bash
ffmpeg -i 83.png -vf crop=600:600:0:0 84.png
```

![4](img/84.png)

求解结果：

![5](img/85.png)

基本上就是无效的了。

> 压缩后的水印大小是 64x65：
>
> ```text
> file 820.png 
> 820.png: PNG image data, 64 x 65, 8-bit/color RGB, non-interlaced
> ```
>
> 在求解的时候，`wm_shape=(65, 64)`得到了上图几乎看不见有效信息的结果，但是如果倒过来：`wm_shape=(64, 65)` 反而可以看到国科大校徽的轮廓（虽然被压扁了）：
> ![0](img/831.png)
> 我没学过信号与系统，我不是很理解这是为什么。

#### 2. 电子包浆——压缩

```bash
$ ffmpeg -i 83.png -vf scale=600:-1 86.png
$ file 83.png 
83.png: PNG image data, 638 x 656, 8-bit/color RGB, non-interlaced
$ file 86.png 
86.png: PNG image data, 600 x 617, 8-bit/color RGB, non-interlaced
```

再次执行：

![6](img/87.png)

能看到极其微弱的轮廓。

#### 3. 电子包浆——格式转换

选取 png 以外的常见图片压缩格式 jpg 和 gif，并且使用 convert 命令转换：

```bash
$ convert 83.png 87.jpg
$ convert 87.jpg 88.gif
$ convert 88.gif 89.png
```

提取：

![89](img/811.png)

这个是看起来最好的，但是仍然不如直接提取。

总结：**仅针对我的这个例子**，裁剪和压缩能够明显影响提取结果，而格式转换的影响较小。总的来说并不如作者在 GitHub 上提供的例子那么准确。这可能和原图大小不大、压缩后的水印不清晰、图片不是黑白的、压缩过程中比例不是很好的比例有关系。其他可能的攻击方式还有角度旋转、手机拍屏等。

## 选做题

按照上面讲义中频域水印的思想，使用 Matlab 或 Python 实现简单的频域水印算法，与实验报告一起提交（至少需要提供一个示例，证明你的实现是有效的）

首先我们可以考虑一下轮子的代码，我的代码在 `~/.local/lib/python3.11/site-packages/blind_watermark` 里，可见我们调用的对应函数主要也是在 `bmw_core.py` 里使用了 np 变换和随机函数实现的。

可以尝试使用 MATLAB 2019B 运行：

```matlab
%%加水印
img = double(imread('81.jpg'));
wm = double(imread('82.png'));

fimgre = fft2(img) + wm*10;
imgre = uint8(real(ifft2(fimgre)));

imwrite(imgre,'813.png');

%%解水印
img = double(imread('81.jpg'));
img_watermark = double(imread('813.png'));
wm = fft2(img_watermark) - fft2(img);

imwrite(mat2gray(real(wm)/10),'814.png');
```

首先该算法要求输入图像和水印尺寸相同。这里我们仍采取中科大校徽为主，用国科大校徽作为水印，需要扩大国科大校徽尺寸：

```bash
ffmpeg -i 082.png  -vf "pad=638:656:0:0:white" 82.png
```

![9](img/812.png)

运行结果：

- 加水印的图：

  ![11](img/813.png)

- 解密的水印：

  ![12](img/814.png)

可见解密水印上，虽然对比度不高，但是还是有清晰的轮廓。
